import React, { useState, useRef, ChangeEvent, useEffect } from "react";
import {
  AreaChart,
  Tooltip,
  Area,
  ResponsiveContainer,
  XAxis,
  YAxis,
} from "recharts";
import { getDownloadURL, ref, uploadBytesResumable } from "firebase/storage";
import moment from "moment";
import ReactPlayer from "react-player";
import { OnProgressProps } from "react-player/base";
import CommonAPI from "../../common/apis/common";
import {
  ApplicationPayloadType,
  ApplicationStatus,
  EmotionsPayloadType,
  EmotionsType,
} from "../../common/types";
import Progress from "../Progress";
import { fileStorage } from "../../common/config";
import { useDispatch } from "react-redux";
import { getJobs } from "../../common/actions/common";
import { updateApplication } from "../../common/actions/application";
import { getVerificationColor } from "../../common/lib/util";
import ApplicationActions from "./ApplicationActions";
import MeetingsAPI from "../../common/apis/meetings";

type OwnProps = {
  application: ApplicationPayloadType;
  status?: ApplicationStatus;
  onChangeStatus: (e: ChangeEvent<HTMLSelectElement>) => void;
  onUpdateStatus: () => void;
};

type Analysis = {
  voice?: string;
  eyeBlinks?: number;
};

const Analyse = ({
  application,
  status,
  onChangeStatus,
  onUpdateStatus,
}: OwnProps) => {
  const dispatch = useDispatch();
  const [analysis, setAnalysis] = useState<Analysis | null>(null);
  const [isLoading, setIsLoading] = useState<boolean>(true);
  const [startTime, setStartTime] = useState<number>(0);
  const [progress, setProgress] = useState<number>(0);
  const [endTime, setEndTime] = useState<number>(120);
  const [emotions, setEmotions] = useState<EmotionsPayloadType[]>([]);
  const [paddingLeft, setPaddingLeft] = useState<number>(0);
  const [isFacialAnalyzing, setIsFacialAnalyzing] = useState<boolean>(false);
  const _selectedTime = useRef<number>(0);
  const _video = useRef<ReactPlayer>(null);
  const _hiddenFileInput = useRef<HTMLInputElement>(null);

  useEffect(() => {
    dispatch(getJobs());
    getRecording();
  }, [dispatch]);

  const getRecording = () => {
    if (application.interview?.link) {
      MeetingsAPI.getRecordings(application.interview.link).then((res: any) => {
        if (res.data?.[0]?.file?.fileUrl) {
          updateUserApplication(res.data?.[0]?.file?.fileUrl);
        } else {
          setIsLoading(false);
        }
      });
    } else {
      setIsLoading(false);
    }
  };

  const onPressStartFacialExpression = () => {
    setIsFacialAnalyzing(true);
    const url = `ws://127.0.0.1:8000/facial/ws/emotions?application_id=${application._id}`;
    const ws = new WebSocket(url);
    ws.onmessage = (e) => {
      const socketData = JSON.parse(e.data) as EmotionsType;
      if (socketData.end) {
        setIsFacialAnalyzing(false);
        ws.close();
      } else {
        const emotionData: EmotionsPayloadType = socketData.data[0]
          ?.emotions || {
          angry: 0,
          disgust: 0,
          fear: 0,
          happy: 0,
          sad: 0,
          surprise: 0,
          neutral: 0,
        };
        setEmotions((old) => [...old, emotionData]);
        setPaddingLeft((_old) => _old - 40);
      }
    };
    return () => ws.close();
  };

  const onSeek = (e: number) => (_selectedTime.current = e);
  const onProgress = (e: OnProgressProps) =>
    (_selectedTime.current = e.playedSeconds);
  const onSetStartTime = () => setStartTime(_selectedTime.current);
  const onSetEndTime = () => setEndTime(_selectedTime.current);

  const displayStartTime = `${moment.utc(startTime * 1000).format("HH:mm:ss")}`;
  const displayEndTime = `${moment.utc(endTime * 1000).format("HH:mm:ss")}`;

  const onClickAnalyseVoice = () => {
    if (startTime !== endTime) {
      setIsLoading(true);
      CommonAPI.analyseInterview({
        startTime,
        endTime,
        applicationId: application._id,
      })
        .then((res: any) => {
          setAnalysis(res);
          setIsLoading(false);
        })
        .catch((err) => {
          setIsLoading(false);
        });
    }
  };

  const onClickAnalyseBlinks = () => {
    if (startTime !== endTime) {
      setIsLoading(true);
      const url = `ws://127.0.0.1:8000/facial/ws/eye-blinks?application_id=${
        application._id
      }&start=${Math.round(startTime)}&end=${Math.round(endTime)}`;
      const ws = new WebSocket(url);
      ws.onmessage = (e) => {
        const socketData = JSON.parse(e.data) as any;

        if (socketData.end) {
          const mins = (Math.round(endTime) - Math.round(startTime)) / 60;
          const ratio = Math.ceil(socketData.count / mins);
          if (analysis) {
            setAnalysis({ ...analysis, eyeBlinks: ratio });
          } else {
            setAnalysis({ eyeBlinks: ratio });
          }

          setIsLoading(false);
          ws.close();
        }
      };
      return () => ws.close();
    }
  };

  const onSelectVideo = (e: ChangeEvent<HTMLInputElement>) => {
    if (e.target.files) {
      setIsLoading(true);
      const path = `/interviews/${application._id}.mp4`;
      const uploadRef = ref(fileStorage, path);
      const uploadTask = uploadBytesResumable(uploadRef, e.target.files[0]);
      uploadTask.on(
        "state_changed",
        (snapshot) => {
          const percent = Math.round(
            (snapshot.bytesTransferred / snapshot.totalBytes) * 100
          );
          setProgress(percent);
        },
        (err) => {},
        () => {
          getDownloadURL(uploadTask.snapshot.ref).then(updateUserApplication);
        }
      );
    }
  };

  const updateUserApplication = (url: string) => {
    dispatch(
      updateApplication(
        {
          applicationId: application._id,
          candidateId: application.candidate._id || "",
          update: {
            interview: { ...application.interview, videoRef: url },
          },
        },
        () => {
          setProgress(0);
          setIsLoading(false);
        }
      )
    );
  };

  const onClickUpload = () => {
    _hiddenFileInput?.current?.click();
  };

  const renderLoading = isLoading && (
    <div className="spinner-border text-primary" role="status">
      <span className="visually-hidden">Loading...</span>
    </div>
  );

  const renderVoiceVerification = () => {
    if (application.interview?.voiceVerification) {
      if (application.interview.voiceVerification === "Pending") {
        return (
          <strong className="text-primary">Voice verification: Pending</strong>
        );
      }

      const verification = Number(application?.interview?.voiceVerification);
      const score = verification * 100;
      const percentage = `${score?.toFixed(2)}%`;

      return (
        <strong className={getVerificationColor(verification * 100)}>
          Voice verification: {percentage}
        </strong>
      );
    }
    return (
      <strong className="text-primary">Voice verification: Pending</strong>
    );
  };

  if (!application.interview?.videoRef) {
    return (
      <>
        <div className="interview-room">
          <button
            type="button"
            className="btn btn-secondary"
            onClick={onClickUpload}
            disabled={isLoading}
          >
            {isLoading ? "Loading..." : "Upload video"}
          </button>
          <input
            type="file"
            accept="video/mp4"
            ref={_hiddenFileInput}
            onChange={onSelectVideo}
          />
        </div>
        <Progress progress={progress} />
      </>
    );
  }

  return (
    <div>
      <ReactPlayer
        ref={_video}
        url={application.interview.videoRef}
        controls
        height="100%"
        width="100%"
        onSeek={onSeek}
        onProgress={onProgress}
      />
      {renderVoiceVerification()}

      <div className="row" style={{ padding: "12px" }}>
        <div className="col-sm-6">
          <div className="row">
            <button
              type="button"
              className="col-sm-4 btn btn-warning btn-sm"
              onClick={onSetStartTime}
            >
              Start time
            </button>
            <div className="col-sm-8">
              <input
                type="text"
                className="form-control"
                name="startTime"
                disabled
                value={displayStartTime}
              />
            </div>
          </div>
        </div>
        <div className="col-sm-6">
          <div className="row">
            <button
              type="button"
              className="col-sm-4 btn btn-warning btn-sm"
              onClick={onSetEndTime}
            >
              End time
            </button>
            <div className="col-sm-8">
              <input
                type="text"
                className="form-control"
                name="endTime"
                disabled
                value={displayEndTime}
              />
            </div>
          </div>
        </div>
      </div>

      <div className="row mt-2">
        <div className="mb-3 row">
          <div className="col-sm-3">
            <label className="col-form-label">
              <strong>Vocal expression</strong>
            </label>
            <div className="row">
              <div className="col-sm-8">
                <button
                  type="button"
                  className="btn btn-primary w-100 btn-sm"
                  onClick={onClickAnalyseVoice}
                  disabled={isLoading || isFacialAnalyzing}
                >
                  Analyse
                </button>
              </div>
              {renderLoading}
            </div>
          </div>
          <label className="col-sm-3 col-form-label">{analysis?.voice}</label>

          <div className="col-sm-3 ">
            <label className="col-form-label">
              <strong>Eye blink ratio</strong>
            </label>
            <div className="row">
              <div className="col-sm-8">
                <button
                  type="button"
                  className="btn btn-primary w-100 btn-sm"
                  onClick={onClickAnalyseBlinks}
                  disabled={isLoading || isFacialAnalyzing}
                >
                  Analyse
                </button>
              </div>
              {renderLoading}
            </div>
          </div>
          <label className="col-sm-3 col-form-label">
            <strong>{analysis?.eyeBlinks} (blinks per min)</strong>
          </label>
        </div>
      </div>

      <label className="col-sm-2 col-form-label">
        <strong>Facial expressions</strong>
      </label>
      {emotions.length === 0 ? (
        <div className="interview-room">
          <button
            type="button"
            className="btn btn-secondary"
            onClick={onPressStartFacialExpression}
            disabled={isLoading}
          >
            Start analysis
          </button>
        </div>
      ) : (
        <ResponsiveContainer
          height={280}
          width="100%"
          key={`rc_${emotions.length}`}
        >
          <AreaChart data={emotions} key={`ac_${emotions.length}`}>
            <XAxis dataKey="Name" padding={{ left: paddingLeft }} />
            <YAxis />
            <Tooltip isAnimationActive={false} />
            <Area
              type="monotone"
              dataKey="angry"
              stroke="red"
              fillOpacity={0.2}
              fill="red"
              key={`angry_${emotions.length}`}
              isAnimationActive={false}
            />
            <Area
              type="monotone"
              dataKey="disgust"
              stroke="blue"
              fillOpacity={0.2}
              fill="blue"
              key={`disgust_${emotions.length}`}
              isAnimationActive={false}
            />
            <Area
              type="monotone"
              dataKey="fear"
              stroke="black"
              fillOpacity={0.2}
              fill="black"
              key={`fear_${emotions.length}`}
              isAnimationActive={false}
            />
            <Area
              type="monotone"
              dataKey="happy"
              stroke="yellow"
              fillOpacity={0.2}
              fill="yellow"
              key={`happy_${emotions.length}`}
              isAnimationActive={false}
            />
            <Area
              type="monotone"
              dataKey="sad"
              stroke="gray"
              fillOpacity={0.2}
              fill="gray"
              key={`gray_${emotions.length}`}
              isAnimationActive={false}
            />
            <Area
              type="monotone"
              dataKey="surprise"
              stroke="purple"
              fillOpacity={0.2}
              fill="purple"
              key={`purple_${emotions.length}`}
              isAnimationActive={false}
            />
            <Area
              type="monotone"
              dataKey="neutral"
              stroke="brown"
              fillOpacity={0.2}
              fill="brown"
              key={`neutral_${emotions.length}`}
              isAnimationActive={false}
            />
          </AreaChart>
        </ResponsiveContainer>
      )}
      <hr />
      <ApplicationActions
        status={status}
        onChangeStatus={onChangeStatus}
        onUpdateStatus={onUpdateStatus}
        visible={status === "Analyse"}
      />
    </div>
  );
};

export default Analyse;
