import React, { useState, useRef, ChangeEvent } from "react";
import { CreateCompletionResponseChoicesInner } from "openai";
import OpenAPI from "../../common/lib/openApi";
import ApplicationActions from "./ApplicationActions";
import { ApplicationPayloadType, ApplicationStatus } from "../../common/types";
import { useDispatch } from "react-redux";
import { createMeeting, endMeeting } from "../../common/actions/application";
import Meeting from "../Meeting";

type OwnProps = {
  status?: ApplicationStatus;
  onChangeStatus: (e: ChangeEvent<HTMLSelectElement>) => void;
  onUpdateStatus: () => void;
  application: ApplicationPayloadType;
};

const Interview = ({
  status,
  onChangeStatus,
  onUpdateStatus,
  application,
}: OwnProps) => {
  const dispatch = useDispatch();
  const [isLoading, setIsLoading] = useState<boolean>(false);
  const [roomId, setRoomId] = useState<string | null>(null);
  const [questions, setQuestions] = useState<
    CreateCompletionResponseChoicesInner[]
  >([]);
  const subject = useRef<string>("");
  const level = useRef<string>("easy");

  const onPressGenerate = () => {
    if (subject.current.length > 0) {
      setQuestions([]);
      setIsLoading(true);
      OpenAPI.getInterViewQuestions(subject.current, level.current).then(
        (res) => {
          setQuestions(res.data.choices);
          setIsLoading(false);
        }
      );
    }
  };

  const onChange = (e: ChangeEvent<HTMLInputElement>) => {
    subject.current = e.target.value;
  };

  const onSelectLevel = (e: ChangeEvent<HTMLSelectElement>) => {
    level.current = e.target.value;
  };

  const renderQuestions = (
    question: CreateCompletionResponseChoicesInner,
    index: number
  ) => {
    return (
      <p key={index} style={{ whiteSpace: "pre-line" }}>
        {question.text}
      </p>
    );
  };

  const btnText = isLoading ? "Generating.." : "Generate";

  const onClickStart = () => {
    dispatch(
      createMeeting(application, (_roomId) => {
        setRoomId(_roomId);
      })
    );
  };

  const onMeetingLeave = () => {
    if (roomId) {
      dispatch(
        endMeeting({ application, roomId }, () => {
          setRoomId(null);
        })
      );
    }
  };

  return (
    <div>
      <h4>Interview Room</h4>

      {roomId ? (
        <div className="interview-room">
          <Meeting roomId={roomId} onMeetingLeave={onMeetingLeave} />
        </div>
      ) : (
        <div className="interview-room">
          <div>
            <p className="mb-2">{`Interview is scheduled on ${application.interview?.date} @ ${application.interview?.time}`}</p>
            <button
              type="button"
              className="btn btn-secondary"
              disabled={isLoading}
              onClick={onClickStart}
            >
              Start now
            </button>
          </div>
        </div>
      )}

      <hr />
      <h4>Questions Generator</h4>
      <div className="row g-3">
        <label className="col-3 col-form-label">Subject</label>
        <div className="col-6">
          <div className="row g-3">
            <div className="col-8">
              <input type="text" className="form-control" onChange={onChange} />
            </div>
            <div className="col-4">
              <select
                className="form-select"
                aria-label="Default select example"
                onChange={onSelectLevel}
              >
                <option value="easy">Easy</option>
                <option value="intermediate">Medium</option>
                <option value="hard">Hard</option>
              </select>
            </div>
          </div>
        </div>
        <div className="col-3">
          <button
            className="btn btn-warning mb-3"
            onClick={onPressGenerate}
            disabled={isLoading}
          >
            {btnText}
          </button>
        </div>
      </div>
      <div className="questions-container">
        {questions.map(renderQuestions)}
      </div>
      <hr />
      <ApplicationActions
        status={status}
        onChangeStatus={onChangeStatus}
        onUpdateStatus={onUpdateStatus}
        visible={status === "In progress"}
      />
    </div>
  );
};

export default Interview;
